#!/usr/bin/env python
# coding: utf-8

from setuptools import setup

setup(
    name="pboss",
    version='0.0.1',
    author='u2400',
    url="https://pypi.python.org/pypi/pboss",
    download_url="https://pypi.python.org/pypi/pboss",
    description=u'',
    packages=[''],
    install_requires=[],
    entry_points={}
)